#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Jandy Aqualink Plugin
# Developed by J. Yergey - Sep-2011

################################################################################
# Python imports
import string

# local imports
from jandyaqualink import JandyAqualink


##################################################################################################
class Plugin(indigo.PluginBase):

	##############################################################################################
	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
		# Call the base class's init method
		indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
		# Setting debug to True will show you verbose debugging information in the Indigo Event Log
		# LOTS of debugLog commands are placed throughout this and Jandyqualink routine.
		# These can be activated to "debug" misbahavior between Indigo <-> Aqualink Serial <-> Jandy System.
		self.debug = False
		self.jandyaqualink = JandyAqualink(self)

		
	########################################
	def __del__(self):
		indigo.PluginBase.__del__(self)
		
	##############################################################################################
	# Built-in control methods
	##############################################################################################
	def startup(self):
		# Nothing to do here - it's all taken care of in the deviceStart/StopComm methods
		pass
	
	########################################
	def shutdown(self):
		# Nothing to do since deviceStopComm will be called for the
		# Jandy device and that's how we'll clean up
		pass


	#########################################
	### Validate that appropriate selections were made in device creation / modification dialogs.
	### Ensure there is a valid serial port.
	def validateDeviceConfigUi(self, valuesDict, deviceTypeId, devId):		
		if len(valuesDict["serialPort"]) == 0:
			# User has not selected a valid serial port -- show an error.
			errorMsgDict = indigo.Dict()
			errorMsgDict["serialPort"] = "Select a valid serial port. If none are listed, then make sure you have installed the FTDI VCP driver."
			return (False, valuesDict, errorMsgDict)
	### Check to see if there are already devices created for this plugin using the same serial port.
	###	If using same serial port, probably just a mistake, otherwise note that multiple devices are not supported.
		for dev in indigo.devices.iter("self"):
			if devId != dev.id:				# Only check if this is not an existing device.
				devProps = dev.pluginProps
				priorPortName = devProps.get("serialPort")
				currentPortName = valuesDict["serialPort"]
				if priorPortName == currentPortName:
					errorMsgDict = indigo.Dict()
					errorMsgDict["serialPort"] = "Device " + dev.name + " already set to use this port.  Can't have two devices on same serial port."
					return (False, valuesDict, errorMsgDict)
				else:
					errorMsgDict = indigo.Dict()
					errorMsgDict["serialPort"] = "Can't have two devices with this plugin; please CANCEL device setup."
					return (False, valuesDict, errorMsgDict)				
	### User choices look good, so return True (client will then close the dialog window).
		return (True, valuesDict)


	######################
	def getDeviceStateList(self, dev):
		typeId = dev.deviceTypeId
		devId = dev.id
	###	True, and necessary, for new device, when plugin type is selected.
		if typeId not in self.devicesTypeDict:
			return None

	###	True, and necessary, for new device, when device model is selected.
		defaultStatesList = self.devicesTypeDict[typeId]["States"]
		if devId not in indigo.devices:
			return defaultStatesList

	### Make sure user didn't change device from X10/INSTEON to plugin
		if dev.pluginId != self.pluginId:
			return defaultStatesList

	###	Jandy device states set as default/non-configurable based on Model include:
	###		pumpState, valveState, poolHeat, poolSetpoint, poolTemp, spaHeat, spaSetpoint, spaTemp,
	###		tempUnits, airTemp, commActive and loggingChoice
	###	All auxiliary options are "activated," i.e. stored in state list, based on respective checkbox.
	###	Generic AUX states, if "active," can also have user selected labels, but state IDs remain "aux."
	### "Activate" solar temperature state (solTemp) based on checkbox for solar heater (solHeat).

		### Generate key to checkbox ID, and parse state "activation" based on checkbox selections.
		###	For those states without checkboxes, store default state information (non-configurable).
		###	First, set number of AUX to parse, other than AUXX; RS-4 = 3, RS-6 & RS2/6 = 6, RS-8 = 7, respectively
		devProps = dev.pluginProps
		if typeId != "jandyAqualink-RS-4-Combo":
			numAux = 6
		elif typeId == "jandyAqualink-RS-8-Combo":
				numAux = 7
		else:				# Must be RS-4 unit.
			numAux = 3
		newStatesList = []
		for state in defaultStatesList:
			checkboxKey = state["Key"] + "Checkbox"
			###	AUX1 is AUX1 or CLEANR, not both
			if checkboxKey == "aux1Checkbox":
				if devProps["aux1Checkbox"] == True:
					if devProps["cleanrCheckbox"] != True:
						labelKey = state["Key"] + ".label"
						if labelKey in devProps:
							auxLabel = devProps[labelKey]
							state["TriggerLabel"] = auxLabel
							state["StateLabel"] = auxLabel
						newStatesList.append(state)					
			elif checkboxKey == "cleanrCheckbox":
				if devProps["aux1Checkbox"] == True:
					if devProps["cleanrCheckbox"] == True:
						newStatesList.append(state)
			###	AUX2 is AUX2 or PULMPLO, not both
			elif checkboxKey == "aux2Checkbox":
				if devProps["aux2Checkbox"] == True:
					if devProps["pumpLoCheckbox"] != True:
						labelKey = state["Key"] + ".label"
						if labelKey in devProps:
							auxLabel = devProps[labelKey]
							state["TriggerLabel"] = auxLabel
							state["StateLabel"] = auxLabel
						newStatesList.append(state)
			elif checkboxKey == "pumpLoCheckbox":
				if devProps["aux2Checkbox"] == True:
					if devProps["pumpLoCheckbox"] == True:
						newStatesList.append(state)
			###	AUX3 is AUX3 or WFALL, not both
			elif checkboxKey == "aux3Checkbox":
				if devProps["aux3Checkbox"] == True:
					if devProps["wfallCheckbox"] != True:
						labelKey = state["Key"] + ".label"
						if labelKey in devProps:
							auxLabel = devProps[labelKey]
							state["TriggerLabel"] = auxLabel
							state["StateLabel"] = auxLabel
						newStatesList.append(state)
			elif checkboxKey == "wfallCheckbox":
				if devProps["aux3Checkbox"] == True:
					if devProps["wfallCheckbox"] == True:
						newStatesList.append(state)
			###	AUX 4, if available on system
			elif checkboxKey == "aux4Checkbox" and numAux >= 4:
				if devProps["aux4Checkbox"] == True:
					labelKey = state["Key"] + ".label"
					if labelKey in devProps:
						auxLabel = devProps[labelKey]
						state["TriggerLabel"] = auxLabel
						state["StateLabel"] = auxLabel
					newStatesList.append(state)
			###	AUX 5, if available on system
			elif checkboxKey == "aux5Checkbox" and numAux >= 5:
				if devProps["aux5Checkbox"] == True:
					labelKey = state["Key"] + ".label"
					if labelKey in devProps:
						auxLabel = devProps[labelKey]
						state["TriggerLabel"] = auxLabel
						state["StateLabel"] = auxLabel
					newStatesList.append(state)
			###	AUX 6, if available on system
			elif checkboxKey == "aux6Checkbox" and numAux >= 6:
				if devProps["aux6Checkbox"] == True:
					labelKey = state["Key"] + ".label"
					if labelKey in devProps:
						auxLabel = devProps[labelKey]
						state["TriggerLabel"] = auxLabel
						state["StateLabel"] = auxLabel
					newStatesList.append(state)
			###	AUX 7, if available on system
			elif checkboxKey == "aux7Checkbox" and numAux == 7:
				if devProps["aux7Checkbox"] == True:
					labelKey = state["Key"] + ".label"
					if labelKey in devProps:
						auxLabel = devProps[labelKey]
						state["TriggerLabel"] = auxLabel
						state["StateLabel"] = auxLabel
					newStatesList.append(state)
			###	AUXX is AUXX or Solar Heater, not both
			### For solar heater, need to parse all versions of state
			### If solar heater is "active," then so is solar temperature (solTemp)
			elif checkboxKey == "auxXCheckbox":
				if devProps["auxXCheckbox"] == True:
					if devProps["solHeatCheckbox"] != True:
						labelKey = state["Key"] + ".label"
						if labelKey in devProps:
							auxLabel = devProps[labelKey]
							state["TriggerLabel"] = auxLabel
							state["StateLabel"] = auxLabel
						newStatesList.append(state)
			elif checkboxKey == "solHeatCheckbox":
				if devProps["auxXCheckbox"] == True:
					if devProps["solHeatCheckbox"] == True:
						newStatesList.append(state)
			elif checkboxKey == "solHeat.onCheckbox":
				if devProps["auxXCheckbox"] == True:
					if devProps["solHeatCheckbox"] == True:
						newStatesList.append(state)
			elif checkboxKey == "solHeat.offCheckbox":
				if devProps["auxXCheckbox"] == True:
					if devProps["solHeatCheckbox"] == True:
						newStatesList.append(state)
			elif checkboxKey == "solHeat.enabledCheckbox":
				if devProps["auxXCheckbox"] == True:
					if devProps["solHeatCheckbox"] == True:
						newStatesList.append(state)
			elif checkboxKey == "solTempCheckbox":
				if devProps["auxXCheckbox"] == True:
					if devProps["solHeatCheckbox"] == True:
						newStatesList.append(state)
			### Finally, store those "states" that are non-configurable, i.e. do not have checkboxes on the set-up screen.
			else:
				newStatesList.append(state)
		return newStatesList


	#########################################
	def deviceStartComm(self, dev):
		self.debugLog("<<-- entering deviceStartComm, Device: " + dev.name + "; ID=" +  str(dev.id) + ", Type=" + dev.deviceTypeId)
		self.jandyaqualink.startCommThread(dev)
		self.debugLog(u"exiting deviceStartComm -->>")

	def deviceStopComm(self, dev):
		self.debugLog("<<-- entering deviceStopComm, Device: " + dev.name + "; ID=" +  str(dev.id) + ", Type=" + dev.deviceTypeId)
		self.jandyaqualink.stopCommThread(dev)
		self.debugLog("exiting deviceStopComm -->>")


	##############################################################################################
	# Actions object callbacks 
	##############################################################################################

	#################################
	#	General Actions
	#
	### Queue a direct Jandy command; use carefully, as there is minimal syntax checking
	def directJandy(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		command = pluginAction.props.get("newCommand")
		### Convert commands to upper case and remove any spaces; makes checking simpler.
		command = string.upper(command)
		command = string.replace(command, " ", "")	
		### Certain commands should not be allowed, as they can break ability to communicate.
		if string.find(command,"ECHO") != -1:
			indigo.server.log("ECHO command not allowed; must be defaulted to 0 for proper communication.")			
		elif string.find(command,"RSPFMT") != -1:
			indigo.server.log("RSPFMT command is not allowed;  must be defaulted to 0 for proper communication.")			
		elif string.find(command,"CMDCHR") != -1:
			indigo.server.log("CMDCHR command is not allowed;  must be defaulted to 35 = # for proper communication.")			
		elif string.find(command,"NRMCHR") != -1:
			indigo.server.log("NRMCHR command is not allowed;  must be defaulted to 33 = ! for proper communication.")			
		elif string.find(command,"ERRCHR") != -1:
			indigo.server.log("ERRCHR command is not allowed;  must be defaulted to 63 = ? for proper communication.")			
		elif command == []:
			indigo.server.log("No command was entered; specify in Edit Action Settings.")						
		### Undocumented action to switch for plugin detailed debugging.
		elif command == "DEBUG":
			if self.debug == True:
				self.debug = False
			else:
				self.debug = True		
		### Undocumented action to Log states, their current values, and device properties - used for debugging
		#	Just log C or F for temp units state, and log AUX labels along with current state
		elif command == "LOGSTATES":
			for state in dev.states:
				currentState = dev.states[state]
				if state == "tempUnits":
					currentState = currentState[1]
				if state[:3] == "aux":
					devProps = dev.pluginProps
					auxLabel = devProps[state + ".label"]
					indigo.server.log(state + ", " + auxLabel + ", " + str(currentState))
				else:
					indigo.server.log(state + ", " + str(currentState))
			for prop in dev.pluginProps:
				currentProperty = dev.pluginProps[prop]
				indigo.server.log(prop + ", " + str(currentProperty))
		elif command[0] != "#":
			indigo.server.log("Command must start with # symbol; re-enter in Edit Action Settings.")			
		else:
			self.jandyaqualink.queueSerialCmd(command)
			if dev.states["loggingChoice"] != "logOff":
				indigo.server.log(command + " command queued.")
	### Set Jandy state change logging choice, used to provide feedback on Jandy commands; NOT the same as debugging.
	def setLoggingChoice(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		newState = pluginAction.props.get("loggingChoice")
		if newState == "logOff":
			dev.updateStateOnServer("loggingChoice", value="logOff")
			indigo.server.log("Logging of Jandy state changes turned off.")
		elif newState == "logAllButTemp":
			dev.updateStateOnServer("loggingChoice", value="logAllButTemp")
			indigo.server.log("Logging of Jandy state changes set to log all but temperature changes.")
		elif newState == "logAll":
			dev.updateStateOnServer("loggingChoice", value="logAll")
			indigo.server.log("Logging of Jandy state changes set to log all changes.")
		else:
			indigo.server.log("No selection was made for logging; specify in Edit Action Settings.")			
	### Toggle Jandy state change logging choice, used to provide feedback on Jandy commands; NOT the same as debugging.
	def toggleLoggingChoice(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		currentState = dev.states["loggingChoice"]
		if currentState == "logOff":
			dev.updateStateOnServer("loggingChoice", value="logAllButTemp")
			indigo.server.log("Logging of Jandy state changes set to log all but temperature changes.")
		elif currentState == "logAllButTemp":
			dev.updateStateOnServer("loggingChoice", value="logAll")
			indigo.server.log("Logging of Jandy state changes set to log all changes.")
		else:
			dev.updateStateOnServer("loggingChoice", value="logOff")
			indigo.server.log("Logging of Jandy state changes turned off.")
	
	#################################
	#	Valve Switching Action (Only applicable for Pool/Spa COMBO systems)
	#
	### Switch valves, if pump is on and valves not switching, and valves not already set as requested.
	def switchValves(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		typeId = dev.deviceTypeId
		if typeId == "jandyAqualink-RS-2-6-SepEquip":
			indigo.server.log("Valves switching available for Pool/Spa Combo systems only.")			
		else:	
			currentState = dev.states["pumpState"]
			if currentState == "off":
				indigo.server.log("Pump not on; can't switch valves.")
			elif currentState == "waitingForDelay":
				indigo.server.log("Pump is already waiting for delay; can't switch valves.")
			else:
				currentState = dev.states["valveState"]
				newState = pluginAction.props.get("valvePopUp")
				if currentState == "pool" and newState == "pool":
					indigo.server.log("Valves already set to Pool.")			
				elif currentState == "spa" and newState == "spa":
					indigo.server.log("Valves already set to Spa.")			
				elif newState == "spa":
					self.jandyaqualink.queueSerialCmd("#SPA=ON")
				elif newState == "pool":
					self.jandyaqualink.queueSerialCmd("#SPA=OFF")
				else:
					indigo.server.log("No selection was made (pool/spa); specify in Edit Action Settings.")			

	#################################
	#	Pump Toggle Action (Only applicable for Pool/Spa COMBO systems)
	#
	### Toggle pump on/off.
	def togglePump(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		typeId = dev.deviceTypeId
		if typeId == "jandyAqualink-RS-2-6-SepEquip":
			indigo.server.log("Valves switching available for Pool/Spa Combo systems only.")			
		else:	
			currentState = dev.states["pumpState"]
			if currentState == "off":
				self.jandyaqualink.queueSerialCmd("#PUMP=ON")
			elif currentState == "waitingForDelay":
				indigo.server.log("Pump is waiting for delay; can't toggle pump on/off.")
			else:
				self.jandyaqualink.queueSerialCmd("#PUMP=OFF")

	#################################
	#	Pool Pump Toggle Action (Only applicable for Pool/Spa SEPARATE systems)
	#
	### Toggle pool pump on/off.
	def togglePoolPump(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		typeId = dev.deviceTypeId
		if typeId != "jandyAqualink-RS-2-6-SepEquip":
			indigo.server.log("Pool pump toggle available for Pool/Spa Separate systems only.")			
		else:	
			currentState = dev.states["poolPumpState"]
			if currentState == "off":
				self.jandyaqualink.queueSerialCmd("#PUMP=ON")
			else:
				self.jandyaqualink.queueSerialCmd("#PUMP=OFF")

	#################################
	#	Spa Pump Toggle Action (Only applicable for Pool/Spa SEPARATE systems)
	#
	### Toggle pool pump on/off.
	def toggleSpaPump(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		typeId = dev.deviceTypeId
		if typeId != "jandyAqualink-RS-2-6-SepEquip":
			indigo.server.log("Spa pump toggle available for Pool/Spa Separate systems only.")			
		else:	
			currentState = dev.states["spaPumpState"]
			if currentState == "off":
				self.jandyaqualink.queueSerialCmd("#SPA=ON")
			else:
				self.jandyaqualink.queueSerialCmd("#SPA=OFF")

	#################################
	#	Cleaner Toggle Action
	#
	### Toggle cleaner on/off.
	def toggleCleanr(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		currentState = dev.states["cleanr"]
		if currentState == "off":
			self.jandyaqualink.queueSerialCmd("#CLEANR=ON")
		else:
			self.jandyaqualink.queueSerialCmd("#CLEANR=OFF")

	#################################
	#	Waterfall Toggle Action
	#
	### Toggle waterfall on/off.
	def toggleWfall(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		currentState = dev.states["wfall"]
		if currentState == "off":
			self.jandyaqualink.queueSerialCmd("#WFALL=ON")
		else:
			self.jandyaqualink.queueSerialCmd("#WFALL=OFF")

	#################################
	#	Pool / Spa / Solar Heat Actions
	#
	### Turn on heat.
	def turnOnHeat(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		heatChoice = pluginAction.props.get("heatOnPopUp")
		if heatChoice == "pool":
			currentState = dev.states["poolHeat"]
			if currentState == "on":
				indigo.server.log("Pool heat already on.")			
			elif currentState == "enabled":
				indigo.server.log("Pool heat already enabled.")
			else:
				self.jandyaqualink.queueSerialCmd("#POOLHT=ON")
		elif heatChoice == "spa":
			currentState = dev.states["spaHeat"]
			if currentState == "on":
				indigo.server.log("Spa heat already on.")			
			elif currentState == "enabled":
				indigo.server.log("Spa heat already enabled.")
			else:
				self.jandyaqualink.queueSerialCmd("#SPAHT=ON")
####!!!! NOTE: This command/response has not been properly tested.
		elif heatChoice == "solar":
			if "solHeat" in dev.states:
				currentState = dev.states["solHeat"]
				if currentState == "active":
					indigo.server.log("Solar heat already active.")			
				elif currentState == "enabled":
					indigo.server.log("Solar heat already enabled.")
				else:
					self.jandyaqualink.queueSerialCmd("#SOLHT=ON")
			else:
				indigo.server.log("Solar heat not installed on this system.")
		else:
			indigo.server.log("No selection was made (pool/spa/solar); specify in Edit Action Settings.")			
	### Turn off heat.
	def turnOffHeat(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		heatChoice = pluginAction.props.get("heatOffPopUp")
		if heatChoice == "pool":
			currentState = dev.states["poolHeat"]
			if currentState == "off":
				indigo.server.log("Pool heat already off.")			
			else:
				self.jandyaqualink.queueSerialCmd("#POOLHT=OFF")
		elif heatChoice == "spa":
			currentState = dev.states["spaHeat"]
			if currentState == "off":
				indigo.server.log("Spa heat already off.")			
			else:
				self.jandyaqualink.queueSerialCmd("#SPAHT=OFF")
####!!!! NOTE: This command/response has not been properly tested.
		elif heatChoice == "solar":
			if "solHeat" in dev.states:
				currentState = dev.states["solHeat"]
				if currentState == "off":
					indigo.server.log("Solar heat already off.")			
				else:
					self.jandyaqualink.queueSerialCmd("#SOLHT=OFF")
		else:
			indigo.server.log("No selection was made (pool/spa/solar); specify in Edit Action Settings.")
	### Toggle heat.
	def toggleHeat(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		heatChoice = pluginAction.props.get("heatTogglePopUp")
		if heatChoice == "pool":
			currentState = dev.states["poolHeat"]
			if currentState == "off":
				self.jandyaqualink.queueSerialCmd("#POOLHT=ON")			
			else:
				self.jandyaqualink.queueSerialCmd("#POOLHT=OFF")
		elif heatChoice == "spa":
			currentState = dev.states["spaHeat"]
			if currentState == "off":
				self.jandyaqualink.queueSerialCmd("#SPAHT=ON")			
			else:
				self.jandyaqualink.queueSerialCmd("#SPAHT=OFF")
####!!!! NOTE: This command/response has not been properly tested.
		elif heatChoice == "solar":
			if "solHeat" in dev.states:
				currentState = dev.states["solHeat"]
				if currentState == "off":
					self.jandyaqualink.queueSerialCmd("#SOLHT=ON")			
				else:
					self.jandyaqualink.queueSerialCmd("#SOLHT=OFF")
		else:
			indigo.server.log("No selection was made (pool/spa/solar); specify in Edit Action Settings.")

	#################################
	#	Pool / Spa Setpoint Actions
	#
	### New Setpoint.
	def newSetpoint(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		try:
			newSetpoint = int(pluginAction.props.get("newSetpoint"))
			currentUnits = dev.states["tempUnits"]
			if currentUnits[1] == "C":
				if newSetpoint < 15 or newSetpoint > 40:
					indigo.server.log("Temperature setpoints must be between 15°C and 40°C.")
					return
			else:
				if newSetpoint < 60 or newSetpoint > 104:
					indigo.server.log("Temperature setpoints must be between 60°F and 104°F.")
					return			
			setpointChoice = pluginAction.props.get("newSetpointPopUp")
			if setpointChoice == "pool":
				self.jandyaqualink.queueSerialCmd("#POOLSP=" + str(newSetpoint))
			elif setpointChoice == "spa":
				self.jandyaqualink.queueSerialCmd("#SPASP=" + str(newSetpoint))
			else:
				indigo.server.log("No selection was made (pool/spa); specify in Edit Action Settings.")
		except ValueError:
			indigo.server.log("Temperatures must be provided as integer, without ° symbol or F/C.")
	### Increment Setpoint.  Ensure in range using the current temperature units.
	def incrementSetpoint(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		setpointChoice = pluginAction.props.get("incrementSetpointPopUp")
		if setpointChoice == "pool":
			currentSetpoint = dev.states["poolSetpoint"]
		else:
			currentSetpoint = dev.states["spaSetpoint"]
		try:
			newSetpoint = currentSetpoint + int(pluginAction.props.get("incrementSetpoint"))
			currentUnits = dev.states["tempUnits"]
			if currentUnits[1] == "C":
				if newSetpoint < 15 or newSetpoint > 40:
					indigo.server.log("Temperature setpoints must be between 15°C and 40°C.")
					return
			else:
				if newSetpoint < 60 or newSetpoint > 104:
					indigo.server.log("Temperature setpoints must be between 60°F and 104°F.")
					return
			if setpointChoice == "pool":
				self.jandyaqualink.queueSerialCmd("#POOLSP=" + str(newSetpoint))
			elif setpointChoice == "spa":
				self.jandyaqualink.queueSerialCmd("#SPASP=" + str(newSetpoint))
			else:
				indigo.server.log("No selection was made (pool/spa); specify in Edit Action Settings.")
		except ValueError:
			indigo.server.log("Temperature changes must be provided as integer, without ° symbol or F/C.")

	#################################
	#	Auxillary Actions	
	#
	#	NOTE: AUX values are "off" or "on" if not dimmable, and "off", " 25%", " 50%", " 75%", "100%" if dimmable
	#	NOTE: Ensure if auxX is selected that the "X" is upper case, otherwise it won't be recognized.
	### Turn on Aux
	def turnOnAux(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		auxChoice = pluginAction.props.get("auxChoice")
		auxChoice = string.upper(auxChoice)
		if ("aux" + auxChoice) in dev.states:
			auxState = dev.states["aux" + auxChoice]
			if auxState != "off":
				indigo.server.log("AUX" + auxChoice + " already on.")			
			else:
				self.jandyaqualink.queueSerialCmd("#AUX" + auxChoice + "=ON")
		elif auxChoice == "":
			indigo.server.log("No aux selection was made; specify in Edit Action Settings.")
		else:
			indigo.server.log("AUX" + auxChoice + " is not installed; cannot be set.")			
	### Turn off Aux
	def turnOffAux(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		auxChoice = pluginAction.props.get("auxChoice")
		auxChoice = string.upper(auxChoice)
		if ("aux" + auxChoice) in dev.states:
			auxState = dev.states["aux" + auxChoice]
			if auxState != "on":
				indigo.server.log("AUX" + auxChoice + " already off.")			
			else:
				self.jandyaqualink.queueSerialCmd("#AUX" + auxChoice + "=OFF")
		elif auxChoice == "":
			indigo.server.log("No aux selection was made; specify in Edit Action Settings.")
		else:
			indigo.server.log("AUX" + auxChoice + " is not installed; cannot be set.")			
	### Toggle Aux
	def toggleAux(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		auxChoice = pluginAction.props.get("auxChoice")
		auxChoice = string.upper(auxChoice)
		if ("aux" + auxChoice) in dev.states:
			auxState = dev.states["aux" + auxChoice]
			if auxState == "off":
				self.jandyaqualink.queueSerialCmd("#AUX" + auxChoice + "=ON")
			else:
				self.jandyaqualink.queueSerialCmd("#AUX" + auxChoice + "=OFF")
		elif auxChoice == "":
			indigo.server.log("No aux selection was made; specify in Edit Action Settings.")
		else:
			indigo.server.log("AUX" + auxChoice + " is not installed; cannot be set.")			

###!!!	Dimming commands have not been validated.
	### Dim Aux
	def dimAux(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		auxChoice = pluginAction.props.get("auxChoice")
		auxChoice = string.upper(auxChoice)
		if ("aux" + auxChoice) in dev.states:
			auxState = dev.states["aux" + auxChoice]
			if auxState == "off":
				indigo.server.log("AUX" + auxChoice + " already 0% = off, can't dim.")			
			else:
				self.jandyaqualink.queueSerialCmd("#AUX" + auxChoice + "-")
		elif auxChoice == "":
			indigo.server.log("No aux selection was made; specify in Edit Action Settings.")
		else:
			indigo.server.log("AUX" + auxChoice + " is not installed; cannot be set.")			
	### Brighten Aux
	def brightenAux(self, pluginAction):
		dev = indigo.devices[pluginAction.deviceId]
		auxChoice = pluginAction.props.get("auxChoice")
		auxChoice = string.upper(auxChoice)
		if ("aux" + auxChoice) in dev.states:
			auxState = dev.states["aux" + auxChoice]
			if auxState == "100%":
				indigo.server.log("AUX" + auxChoice + " already 100%, can't brighten.")			
			else:
				self.jandyaqualink.queueSerialCmd("#AUX" + auxChoice + "+")
		elif auxChoice == "":
			indigo.server.log("No aux selection was made; specify in Edit Action Settings.")
		else:
			indigo.server.log("AUX" + auxChoice + " is not installed; cannot be set.")			


